@@ -25,6 +25,7 @@ module Pipes.Prelude (
       stdinLn
     , readLn
     , fromHandle
+    , repeatM
     , replicateM
 
     -- * Consumers
@@ -179,8 +180,22 @@ fromHandle h = go
             go
 {-# INLINABLE fromHandle #-}
 
--- | Repeat a monadic action a fixed number of times, 'yield'ing each result
-replicateM :: Monad m => Int -> m a -> Producer a m ()
+-- | Repeat a monadic action indefinitely, 'yield'ing each result
+repeatM :: Monad m => m a -> Producer' a m r
+repeatM m = lift m >~ cat
+{-# INLINABLE repeatM #-}
+
+{-# RULES
+  "repeatM m >-> p" forall m p . repeatM m >-> p = lift m >~ p
+  #-}
+
+{-| Repeat a monadic action a fixed number of times, 'yield'ing each result
+
+> replicateM  0      x = return ()
+>
+> replicateM (m + n) x = replicateM m x >> replicateM n x  -- 0 <= {m,n}
+-}
+replicateM :: Monad m => Int -> m a -> Producer' a m ()
 replicateM n m = lift m >~ take n
 {-# INLINABLE replicateM #-}
 
@@ -258,7 +273,12 @@ quit<Enter>
 
 -}
 
--- | Apply a function to all values flowing downstream
+{-| Apply a function to all values flowing downstream
+
+> map id = cat
+>
+> map (g . f) = map f >-> map g
+-}
 map :: Monad m => (a -> b) -> Pipe a b m r
 map f = for cat (\a -> yield (f a))
 {-# INLINABLE map #-}
@@ -271,7 +291,12 @@ map f = for cat (\a -> yield (f a))
         return (f a) ) >~ p
   #-}
 
--- | Apply a monadic function to all values flowing downstream
+{-| Apply a monadic function to all values flowing downstream
+
+> mapM return = cat
+>
+> mapM (f >=> g) = mapM f >-> mapM g
+-}
 mapM :: Monad m => (a -> m b) -> Pipe a b m r
 mapM f = for cat $ \a -> do
     b <- lift (f a)
@@ -306,7 +331,12 @@ mapFoldable f = for cat (\a -> each (f a))
         p >-> mapFoldable f = for p (\a -> each (f a))
   #-}
 
--- | @(filter predicate)@ only forwards values that satisfy the predicate.
+{-| @(filter predicate)@ only forwards values that satisfy the predicate.
+
+> filter (pure True) = cat
+>
+> filter (liftA2 (&&) p1 p2) = filter p1 >-> filter p2
+-}
 filter :: Monad m => (a -> Bool) -> Pipe a a m r
 filter predicate = for cat $ \a -> when (predicate a) (yield a)
 {-# INLINABLE filter #-}
@@ -318,6 +348,10 @@ filter predicate = for cat $ \a -> when (predicate a) (yield a)
 
 {-| @(filterM predicate)@ only forwards values that satisfy the monadic
     predicate
+
+> filterM (pure (pure True)) = cat
+>
+> filterM (liftA2 (liftA2 (&&)) p1 p2) = filterM p1 >-> filterM p2
 -}
 filterM :: Monad m => (a -> m Bool) -> Pipe a a m r
 filterM predicate = for cat $ \a -> do
@@ -332,7 +366,16 @@ filterM predicate = for cat $ \a -> do
             when b (yield a) )
   #-}
 
--- | @(take n)@ only allows @n@ values to pass through
+{-| @(take n)@ only allows @n@ values to pass through
+
+> take 0 = return ()
+>
+> take (m + n) = take m >> take n
+
+> take <infinity> = cat
+>
+> take (min m n) = take m >-> take n
+-}
 take :: Monad m => Int -> Pipe a a m ()
 take n = replicateM_ n $ do
     a <- await
@@ -341,6 +384,10 @@ take n = replicateM_ n $ do
 
 {-| @(takeWhile p)@ allows values to pass downstream so long as they satisfy
     the predicate @p@.
+
+> takeWhile (pure True) = cat
+>
+> takeWhile (liftA2 (&&) p1 p2) = takeWhile p1 >-> takeWhile p2
 -}
 takeWhile :: Monad m => (a -> Bool) -> Pipe a a m ()
 takeWhile predicate = go
@@ -354,7 +401,12 @@ takeWhile predicate = go
             else return ()
 {-# INLINABLE takeWhile #-}
 
--- | @(drop n)@ discards @n@ values going downstream
+{-| @(drop n)@ discards @n@ values going downstream
+
+> drop 0 = cat
+>
+> drop (m + n) = drop m >-> drop n
+-}
 drop :: Monad m => Int -> Pipe a a m r
 drop n = do
     replicateM_ n await
@@ -363,6 +415,10 @@ drop n = do
 
 {-| @(dropWhile p)@ discards values going downstream until one violates the
     predicate @p@.
+
+> dropWhile (pure False) = cat
+>
+> dropWhile (liftA2 (||) p1 p2) = dropWhile p1 >-> dropWhile p2
 -}
 dropWhile :: Monad m => (a -> Bool) -> Pipe a a m r
 dropWhile predicate = go
@@ -400,7 +456,10 @@ findIndices predicate = loop 0
         loop $! n + 1
 {-# INLINABLE findIndices #-}
 
--- | Strict left scan
+{-| Strict left scan
+
+> Control.Foldl.purely scan :: Monad m => Fold a b -> Pipe a b m r
+-}
 scan :: Monad m => (x -> a -> x) -> x -> (x -> b) -> Pipe a b m r
 scan step begin done = loop begin
   where
@@ -411,7 +470,10 @@ scan step begin done = loop begin
         loop $! x'
 {-# INLINABLE scan #-}
 
--- | Strict, monadic left scan
+{-| Strict, monadic left scan
+
+> Control.Foldl.impurely scan :: Monad m => FoldM a m b -> Pipe a b m r
+-}
 scanM :: Monad m => (x -> a -> m x) -> m x -> (x -> m b) -> Pipe a b m r
 scanM step begin done = do
     x <- lift begin
@@ -425,7 +487,12 @@ scanM step begin done = do
         loop $! x'
 {-# INLINABLE scanM #-}
 
--- | Apply an action to all values flowing downstream
+{-| Apply an action to all values flowing downstream
+
+> chain (pure (return ())) = cat
+>
+> chain (liftA2 (>>) m1 m2) = chain m1 >-> chain m2
+-}
 chain :: Monad m => (a -> m ()) -> Pipe a a m r
 chain f = for cat $ \a -> do
     lift (f a)
@@ -476,7 +543,10 @@ True
 
 -}
 
--- | Strict fold of the elements of a 'Producer'
+{-| Strict fold of the elements of a 'Producer'
+
+> Control.Foldl.purely fold :: Monad m => Fold a b -> Producer a m () -> m b
+-}
 fold :: Monad m => (x -> a -> x) -> x -> (x -> b) -> Producer a m () -> m b
 fold step begin done p0 = loop p0 begin
   where
@@ -487,7 +557,10 @@ fold step begin done p0 = loop p0 begin
         Pure    _     -> return (done x)
 {-# INLINABLE fold #-}
 
--- | Strict, monadic fold of the elements of a 'Producer'
+{-| Strict, monadic fold of the elements of a 'Producer'
+
+> Control.Foldl.impurely foldM :: Monad m => FoldM a b -> Producer a m () -> m b
+-}
 foldM
     :: Monad m
     => (x -> a -> m x) -> m x -> (x -> m b) -> Producer a m () -> m b
