@@ -250,13 +250,13 @@ import Prelude hiding ((.), id)
     signature is just a special case of the second one:
 
 @
- 'for' :: 'Monad' m => 'Producer' a m r -> (a -> 'Producer' b    m ()) -> 'Producer' b    m r
+ 'for' :: 'Monad' m => 'Producer' a m r -> (a -> 'Producer' b m ()) -> 'Producer' b m r
 
 \ -- Specialize \'b\' to \'X\'
  'for' :: 'Monad' m => 'Producer' a m r -> (a -> 'Producer' 'X' m ()) -> 'Producer' 'X' m r
 
 \ -- Producer X = Effect
- 'for' :: 'Monad' m => 'Producer' a m r -> (a -> 'Effect'        m ()) -> 'Effect'        m r
+ 'for' :: 'Monad' m => 'Producer' a m r -> (a -> 'Effect'     m ()) -> 'Effect'     m r
 @
 
     This is the same trick that all @pipes@ functions use to work with various
@@ -438,9 +438,9 @@ import Prelude hiding ((.), id)
 
 @
  (~>) :: Monad m
-      => (a -> 'Producer' b m r)
-      -> (b -> 'Producer' c m r)
-      -> (a -> 'Producer' c m r)
+      => (a -> 'Producer' b m ())
+      -> (b -> 'Producer' c m ())
+      -> (a -> 'Producer' c m ())
  (f ~> g) x = for (f x) g
 @
 
@@ -448,9 +448,9 @@ import Prelude hiding ((.), id)
     into the following more symmetric equation:
 
 @
- f :: Monad m => a -> 'Producer' b m r
- g :: Monad m => b -> 'Producer' c m r
- h :: Monad m => c -> 'Producer' d m r
+ f :: Monad m => a -> 'Producer' b m ()
+ g :: Monad m => b -> 'Producer' c m ()
+ h :: Monad m => c -> 'Producer' d m ()
 
 \ \-\- Associativity
  (f ~> g) ~> h = f ~> (g ~> h)
@@ -568,7 +568,7 @@ import Prelude hiding ((.), id)
 @
 
     One way to feed a 'Consumer' is to repeatedly feed the same input using
-    using ('>~') (pronounced \"feed\"):
+    ('>~') (pronounced \"feed\"):
 
 @
  \-\-                 +- Feed       +- Consumer to    +- Returns new
@@ -1019,7 +1019,7 @@ quit<Enter>
 
 > customerService :: Producer String IO ()
 > customerService = do
->     each [ "Hello, how can I help you?"      -- Begin with a script
+>     each [ "Hello, how can I help you?"        -- Begin with a script
 >          , "Hold for one second."
 >          ]
 >     P.stdinLn >-> P.takeWhile (/= "Goodbye!")  -- Now continue with a human
@@ -1049,10 +1049,11 @@ quit<Enter>
     You can even use 'Enumerable' to traverse effectful types that are not even
     proper containers, like 'Control.Monad.Trans.Maybe.MaybeT':
 
-> input :: MaybeT IO Int
+> input :: MaybeT IO String
 > input = do
 >     str <- lift getLine
 >     guard (str /= "Fail")
+>     return str
 
 >>> runEffect $ every input >-> P.stdoutLn
 Test<Enter>
